from django.conf import settings as django_settings


class Settings(object):
    @property
    def LOCATIONSTREE_LOCATION_DEFAULT_ID(self):
        return getattr(django_settings, 'LOCATIONSTREE_LOCATION_DEFAULT_ID', 1)

    @property
    def LOCATIONSTREE_LOCATION_SESSION_NAME(self):
        return getattr(django_settings, 'LOCATIONSTREE_LOCATION_SESSION_NAME', 'user_location')

    @property
    def LOCATIONSTREE_CONTEXT_VARIABLE_NAME(self):
        return getattr(django_settings, 'LOCATIONSTREE_CONTEXT_VARIABLE_NAME', 'locations')

    @property
    def LOCATIONSTREE_CACHE_TTL(self):
        return getattr(django_settings, 'LOCATIONSTREE_CACHE_TTL', 60*60*24*7)

settings = Settings()



from django import forms
from django.views.generic import FormView
from django.http import HttpResponse
from django.shortcuts import redirect

from .models import Location
from .conf import settings


class DefaultLocationForm(forms.Form):
    location = forms.ModelChoiceField(queryset=Location.objects.all())


class SetUserLocation(FormView):
    template_name = 'locationstree/location_form.html'

    def get_form_class(self):
        return self.kwargs.get('location_form', DefaultLocationForm)

    def form_valid(self, form):
        self.request.session[settings.LOCATIONSTREE_LOCATION_SESSION_NAME] = form.cleaned_data['location'].pk
        if self.request.GET.get('next'):
            return redirect(self.request.GET.get('next'))
        return HttpResponse()

set_user_location = SetUserLocation.as_view()



from django.core.cache import cache

from .models import Location
from .conf import settings


def locations(request):
    locations = cache.get('locationstree.locations')
    if locations is None:
        locations = Location.objects.all()
        cache.set('locationstree.locations', locations, settings.LOCATIONSTREE_CACHE_TTL)
    return {
        settings.LOCATIONSTREE_CONTEXT_VARIABLE_NAME: locations
    }


def session_location(request):
    location_id = request.session.get(settings.LOCATIONSTREE_LOCATION_SESSION_NAME,
                                      settings.LOCATIONSTREE_LOCATION_DEFAULT_ID)
    try:
        location = Location.objects.get(id=location_id)
    except Location.DoesNotExist:
        return {}
    return {'session_location': location}



from django.db import models
from django.utils.translation import ugettext_lazy as _

from mptt.models import MPTTModel, TreeForeignKey


class Location(MPTTModel):
    code = models.CharField(_('Code'), max_length=20, unique=True)
    name = models.CharField(_('Name'), max_length=100)
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children', verbose_name=_('Parent'))
    sort_order = models.IntegerField(_('Sort Order'), default=0)

    class MPTTMeta:
        order_insertion_by = ['sort_order', 'code']

    class Meta:
        verbose_name = _('Location')
        verbose_name_plural = _('Locations')

    def __unicode__(self):
        return self.name



# encoding=UTF-8
import os
from setuptools import setup, find_packages


def read_file(filename):
    """Read a file into a string"""
    path = os.path.abspath(os.path.dirname(__file__))
    filepath = os.path.join(path, filename)
    try:
        return open(filepath).read()
    except IOError:
        return ''


setup(
    name='django-locationstree',
    version=__import__('locationstree').__version__,
    author=u'Witoi.com',
    author_email='dev@witoi.com',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/witoi/django-locationstree',
    description=u' '.join(__import__('locationstree').__doc__.splitlines()).strip(),
    classifiers=[
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Framework :: Django',
        'Development Status :: 1 - Planning',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'Django>=1.4',
        'django-mptt>=0.5.5',
    ],
    long_description=read_file('README.rst'),
    test_suite="runtests.runtests",
    zip_safe=False,
)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Location'
        db.create_table('locationstree_location', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.CharField')(unique=True, max_length=20)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('parent', self.gf('mptt.fields.TreeForeignKey')(blank=True, related_name='children', null=True, to=orm['locationstree.Location'])),
            ('sort_order', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('lft', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('rght', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('tree_id', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('level', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
        ))
        db.send_create_signal('locationstree', ['Location'])


    def backwards(self, orm):
        
        # Deleting model 'Location'
        db.delete_table('locationstree_location')


    models = {
        'locationstree.location': {
            'Meta': {'object_name': 'Location'},
            'code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '20'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['locationstree.Location']"}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        }
    }

    complete_apps = ['locationstree']



from django.core.exceptions import ValidationError
from django.forms.models import ModelChoiceIterator, ModelChoiceField
from django.forms.widgets import Select
from django.utils.encoding import force_unicode
from django.utils.html import escape, conditional_escape
from django.utils.translation import ugettext_lazy as _

from mptt.forms import TreeNodeChoiceField


class JustLeafsLocationField(TreeNodeChoiceField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = SelectWithDisabled()
        self.default_error_messages['not_leaf'] = _('Select a valid choice. %(value)s is not one of the available choices.')
        super(JustLeafsLocationField, self).__init__(*args, **kwargs)

    def _get_choices(self):
        if hasattr(self, '_choices'):
            return self._choices
        return GroupedLocationModelChoiceIterator(self)

    choices = property(_get_choices, ModelChoiceField._set_choices)

    def validate(self, value):
        super(JustLeafsLocationField, self).validate(value)
        if not value.is_leaf_node():
            raise ValidationError(self.error_messages['not_leaf'] % {'value': value})


class GroupedLocationModelChoiceIterator(ModelChoiceIterator):
    def __iter__(self):
        if self.field.empty_label is not None:
            yield (u"", self.field.empty_label)

        for location in self.field.queryset:
            label = self.field.label_from_instance(location)
            if location.is_leaf_node():
                yield (location.id, label)
            else:
                yield (location.id, {'label': label, 'disabled': True})


class SelectWithDisabled(Select):
    """
    Subclass of Django's select widget that allows disabling options.
    To disable an option, pass a dict instead of a string for its label,
    of the form: {'label': 'option label', 'disabled': True}
    """
    def render_option(self, selected_choices, option_value, option_label):
        option_value = force_unicode(option_value)
        if (option_value in selected_choices):
            selected_html = u' selected="selected"'
        else:
            selected_html = ''
        disabled_html = ''
        if isinstance(option_label, dict):
            if dict.get(option_label, 'disabled'):
                disabled_html = u' disabled="disabled"'
            option_label = option_label['label']
        return u'<option value="%s"%s%s>%s</option>' % (
            escape(option_value), selected_html, disabled_html,
            conditional_escape(force_unicode(option_label)))
